#!/usr/bin/perl
##############################################################################
## Monika is a small web interface to monitor PBS node reservations.
## It tries to display a very synthetic view of the cluster usage.
## Monika should work either with PBSPro or OpenPBS.
## Author: pierre.neyron@imag.fr
##############################################################################

use strict;
#use warnings;
use monika::monikaCGI;
use monika::Conf;
use monika::PBS;
use Data::Dumper;

## begin CGI stuff
my ($basename) = $0 =~ /([^\/]+)\.cgi$/;
my $cgi = monikaCGI->new;
my $file = $cgi->param("conf");

## first get nodes description from the configuration file
my $conf = Conf->new;
if ( defined $file and -r $file) {
  $conf->parse($file);
} elsif (-r "./$basename.conf") {
  $conf->parse("./$basename.conf");
} elsif (-r "/etc/$basename.conf") {
  $conf->parse("/etc/$basename.conf");
} else {
  die "Neither /etc/$basename.conf nor ./$basename.conf are readable. I need a configuration file !";
}

## then get nodes description from pbs
my $pbs = PBS->new;
$pbs->pbsnodes($conf->batchscheduler);

$pbs->qstat($cgi);

## my global node container...
my %nodes;

## ... filled in with previousely got nodes:
## first with node descriptions from the configuration file
foreach my $key ( keys %{$conf->allnodes()}) {
  $nodes{$key}=$conf->allnodes->{$key};
}

## then with node descriptions from PBS (which may overwrite descriptions
## from the configuration file, which is what we want for instance if the
## configuration file define a default state (say "missing") for PBS nodes
## that may actually not be described by the pbsnodes cmd...)
foreach my $key ( keys %{$pbs->allnodes()}) {
  $nodes{$key}=$pbs->allnodes->{$key};
}

## set color scheme up
my $colorHash = $conf->colorHash;
while (my ($state,$color) = each %{$colorHash}) {
  $cgi->setColor($state,$color);
}
$cgi->setColorPool($conf->colorPool());

## begin html printing
print $cgi->page_head("Monika: ".$conf->clustername." ".$conf->batchscheduler." nodes");

print $cgi->h1({-align => "center"},
	       $conf->clustername()." ".$conf->batchscheduler." nodes");

## if node param is present, show detailed view of the pointed node
if (defined $cgi->param('node') and defined $nodes{$cgi->param('node')}) {
  my $node = $cgi->param('node');
  print $cgi->h2({-align => "center"},
		 "Node ".$node." detailed status:");
  print $nodes{$node}->htmlStatusTable($cgi);
  print $cgi->h3({ -align => "center" },
		 $cgi->a({ -href => $cgi->self_url(-query=>0)},
		"back to main page"
	      ));
## if job param is present, show detailed view of the pointed job
} elsif (defined $cgi->param('job') and defined $pbs->alljobs()->{$cgi->param('job')}) {
  my $job = $cgi->param('job');
  print $cgi->h2({-align => "center"},
		 "Job ".$job." detailed status:");
  print $pbs->alljobs()->{$job}->htmlStatusTable($cgi);
  print $cgi->h3({ -align => "center" },
		 $cgi->a({ -href => $cgi->self_url(-query=>0)},
		"back to main page"
	      ));
## else show the main page
} else {
  ## print pbs status summary
  print $cgi->h2({-align => "center"},
		 "Summary:");
  print $pbs->htmlSummaryTable($cgi,$conf->batchscheduler);
  print $cgi->br();

  ## print nodes reservations table
  print $cgi->h2({-align => "center"},
		 "Reservations:");
  print $cgi->nodeReservationTable(\%nodes);


  ## print pbs node property chooser
  print $cgi->br();
  print $pbs->htmlPropertyChooser($cgi,$conf->batchscheduler);
  print $pbs->htmlNodeByProperty($cgi);

  ## print pbs job status
  print $cgi->br();
  print $cgi->h2({-align => "center"},
		 "Job details:");
  print $pbs->htmlJobTable($cgi,$conf->batchscheduler);

}

## print  stuff
print $cgi->h6({ -align => "center",
		 onmouseover => "popup('Link description here','yellow')",
		 onmouseout => "kill()"
	       },
	       "- ".
	       $cgi->a({ -href => 'http://ka-tools.sourceforge.net'},
		       "Monika").
	       " - ".
	       $cgi->a({ -href => 'http://www-id.imag.fr'},
		       'ID-Imag').
	       " -"
	      );
print $cgi->end_html();
