## this package handles a PBS job description
package PBSJob;

use monika::monikaCGI;

my $bestEffortColor = "#dddddd";

## class constructor
sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self = {};
  my $jobId = shift;
  defined $jobId or die "missing jobId !";
  $self->{JobId} = $jobId;
  $self->{PBS} = {};
  bless ($self,$class);
  return $self;
}

sub set {
  my $self = shift;
  my $key = shift;
  defined $key or die "which key ?";
  my $value = shift;
  # for best effort jobs
  my $cgi = shift;
  # for best effort jobs
  defined $value or die "which value for $key ?";
  unless (exists $self->{PBS}->{$key}) {
    $self->{PBS}->{$key} = [];
  }
  push @{$self->{PBS}->{$key}},$value;

  # for best effort jobs
  if ($self->get("queue") eq "besteffort"){
    $cgi->setColor($self->numId(),$bestEffortColor);
  }
  # for best effort jobs

  return 1;
}

sub get {
  my $self = shift;
  my $key = shift;
  defined $key or die "which key ?";
  if (exists $self->{PBS}->{$key}) {
    return $self->{PBS}->{$key}->[0];
  } else {
    return undef;
  }
}

sub getList {
  my $self = shift;
  my $key = shift;
  defined $key or die "which key ?";
  exists $self->{PBS}->{$key} or die "Unknown key: ".$key;
  return $self->{PBS}->{$key};
}

sub jobId {
  my $self = shift;
  return $self->{JobId};
}

sub numId {
  my $self = shift;
  $self->jobId() =~ /^(\d+)\./;
  return $1;
}

sub state {
  my $self = shift;
  my %hash = (E => "Exiting",
	      H => "Held",
	      Q => "Queued",
	      R => "Running",
	      T => "Transiting",
	      W => "Waiting");
  my $state = $self->get("job_state");
  if (defined $hash{$state}) {
    return $hash{$state};
  } else {
    return $state;
  }
}

sub owner {
  my $self = shift;
  my $owner = $self->get("Job_Owner");
  $owner =~ s/@.+//;
  return $owner;
}

sub htmlTableRow {
  my $self = shift;
  my $cgi = shift;
  my $batch = shift;
  my $output = "";
  $output .= $cgi->start_Tr();
  $output .= $cgi->colorTd($self->numId(),undef,$cgi->self_url(-query=>0)."?job=".$self->numId(), "click to see job details");
  if (defined $self->get("Mail_Users")) {
    $output .= $cgi->td({-align => "center"},
			$cgi->a({
				 -href => "mailto:".$self->get("Mail_Users"),
				 -title => "click to send mail"
				}, $self->owner()));
  } elsif (Conf::myself()->server_do_mail()) {
    $output .= $cgi->td({-align => "center"},
			$cgi->a({
				 -href => "mailto:".$self->get("Job_Owner"),
				 -title => "click to send mail"
				}, $self->owner()));
  } else {
    $output .= $cgi->td({-align => "center"},$self->owner());
  }
  my @nodecpu = ();
  if (defined $self->get("Resource_List.nodect")) {
    my $nodect = $self->get("Resource_List.nodect");
    push @nodecpu,$nodect.(($nodect > 1)?" nodes":" node");
  }
  if (defined $self->get("Resource_List.ncpus")) {
    my $ncpus = $self->get("Resource_List.ncpus");
    push @nodecpu,$ncpus.(($ncpus > 1)?" CPUs":" CPU");
  }
  if (defined($self->get("Resource_List.ncpus")) || defined($self->get("Resource_List.nodect"))) {
      $output .= $cgi->td({-align => "center"},join " - ",@nodecpu);
  }
  $output .= $cgi->td({-align => "center"},$self->state());
  if ($batch eq "OAR"){
      $output .= $cgi->td({-align => "center"},$self->get("queue"));
      $output .= $cgi->td({-align => "center"},$self->get("nbNodes"));
      $output .= $cgi->td({-align => "center"},$self->get("weight"));
      $output .= $cgi->td({-align => "center"},$self->get("jobType"));
      $output .= $cgi->td({-align => "center"},$self->get("properties"));
      $output .= $cgi->td({-align => "center"},$self->get("reservation"));
      $output .= $cgi->td({-align => "center"},$self->get("walltime"));
      $output .= $cgi->td({-align => "center"},$self->get("submissionTime"));
      $output .= $cgi->td({-align => "center"},$self->get("startTime"));
      $output .= $cgi->td({-align => "center"},$self->get("scheduledStart"));
  }else{
      $output .= $cgi->td( $self->get("comment"));
  }
  $output .= $cgi->end_Tr();
  return $output;
}

sub htmlStatusTable {
  my $self = shift;
  my $cgi = shift;
  my $output = "";
  $output .= $cgi->start_table({-border=>"1",
				 -align => "center"
				});
  $output .= $cgi->start_Tr();
  $output .= $cgi->th({-align => "left", bgcolor => "#c0c0c0"}, $cgi->i("Job Id"));
  $output .= $cgi->th({-align => "left"}, $self->jobId());
  $output .= $cgi->end_Tr();
  my @keylist = keys %{$self->{PBS}};
  foreach my $key (sort @keylist) {
    $output .= $cgi->start_Tr();
    $output .= $cgi->td({-valign => "top", bgcolor=> "#c0c0c0"}, $cgi->i($key));
    my $list = $self->getList($key);
    my $val = join $cgi->br(),@$list;
    $val =~ s/([+,]\s*)/\1<BR>/g;
    $output .= $cgi->td($val);
    $output .= $cgi->end_Tr();
  }
  $output .= $cgi->end_table();
  return $output;
}

return 1;

