## this package handles a PBS node description
package PBSNode;

use strict;
use warnings;
use monika::monikaCGI;
use monika::Conf;
use Data::Dumper;

## class constructor
sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self = {};
  $self->{Nodename} = shift or die "missing a node name !";
  $self->{PBS} = {};
  bless ($self, $class);
  return $self;
}

## return node full name
sub name {
  my $self = shift;
  return $self->{Nodename};
}

## return node state as return by pbsnodes
sub state {
  my $self = shift;
  return $self->get("state");
}

## return this node cpu count
## cpu count seems to be retrived via the "np" property in OpenPbs, but via the pcpus property in PBSPro ???
sub cpus {
  my $self = shift;
  if (defined $self->get("np")) {
    return $self->get("np");
  } elsif (defined $self->get("pcpus")) {
    return $self->get("pcpus");
  }
  return 1;
}

## extract the name to display using a regex from the node real name
sub displayname {
  my $self = shift;
  $self->name() =~ Conf::myself()->nodenameRegex();
  my $shortname = $1 or die "Fail to extract node' shortname";
  return $shortname;;
}

## extract the name to display on the page
sub displayHTMLname {
  my $self = shift;
  $self->name() =~ Conf::myself()->nodenameRegexDisplay();
  my $shortname = $1 or die "Fail to extract node' shortname";
  return $shortname;
}

## return a hash containing (cpu,job) couple.
sub jobs {
  my $self = shift;
  my $jobsStr = $self->get("jobs");
  my %jobs;
  if ($jobsStr) {
    foreach my $jobStr (split /,\s*/, $jobsStr) {
      if ($jobStr =~ /^(\d+)\..+\/(\d+)/) {
	$jobs{$2} = $1;
      } elsif ($jobStr =~ /^(\d+)\/(\d+)\..+/) {
	$jobs{$1} = $2;
      }
    }
  }
  return %jobs;
}

## return the list of properties of this node (PBS resources it is in)
sub properties {
  my $self = shift;
  if ($self->get("properties")) {
    return (split /,/,$self->get("properties"));
  }
  else {
    return ();
  }
}

sub set {
  my $self = shift;
  my $key = shift;
  defined $key or die "which key ?";
  my $value = shift;
  defined $value or die "which value for $key ?";
  unless (exists $self->{PBS}->{$key}) {
    $self->{PBS}->{$key} = [];
  }
  push @{$self->{PBS}->{$key}},$value;
  return 1;
}

sub get {
  my $self = shift;
  my $key = shift;
  defined $key or die "which key ?";
  if (exists $self->{PBS}->{$key}) {
    return $self->{PBS}->{$key}->[0];
  } else {
    return undef;
  }
}

sub getList {
  my $self = shift;
  my $key = shift;
  defined $key or die "which key ?";
  exists $self->{PBS}->{$key} or die "Unknown key: ".$key;
  return $self->{PBS}->{$key};
}

## print this node status HTML table
sub htmlTable {
  my $self = shift;
  my $cgi = shift;
  my $output = "";
  $output .= $cgi->start_table({-border => "1",
				-cellspacing => "0",
				-cellpadding => "0",
				-width => "100%"
			       });
  $output .= $cgi->start_Tr({-align => "center"});
  if ($self->state =~ /down/) {
    $output .= $cgi->colorTd("Down");
  } elsif ($self->state =~ /Absent/) {
    $output .= $cgi->colorTd("Absent");
  } elsif ($self->state =~ /Suspected/) {
    $output .= $cgi->colorTd("Suspected");
  } elsif ($self->state eq "free" or $self->state =~ /job/) {
    my %jobs = $self->jobs;
    for (my $i=0;$i<$self->cpus;$i++) {
      if (defined $jobs{$i}) {
	$output .= $cgi->colorTd($jobs{$i},100/$self->cpus."%");
      } else {
	$output .= $cgi->colorTd("Free",100/$self->cpus."%");
      }
    }
  } else {
    $output .= $cgi->colorTd("Down");
  }
  $output .= $cgi->end_Tr();
  $output .= $cgi->end_table();
  return $output;
}

sub htmlStatusTable {
  my $self = shift;
  my $cgi = shift;
  my $output = "";
  $output .= $cgi->start_table({-border=>"1",
				 -align => "center"
				});
  $output .= $cgi->start_Tr();
  $output .= $cgi->th({-align => "left", bgcolor => "#c0c0c0"}, $cgi->i("Nodename"));
  $output .= $cgi->th({-align => "left"}, $self->name());
  $output .= $cgi->end_Tr();
  my @keylist = keys %{$self->{PBS}};
  foreach my $key (sort @keylist) {
    $output .= $cgi->start_Tr();
    $output .= $cgi->td({-valign => "top", bgcolor => "#c0c0c0"}, $cgi->i($key));
    my $list = $self->getList($key);
    my $val = join $cgi->br(),@$list;
    $val =~ s/([+,]\s*)/$1<BR>/g;
    $output .= $cgi->td($val);
    $output .= $cgi->end_Tr();
  }
  $output .= $cgi->end_table();
  return $output;
}

## that's all.
return 1;
